#include <iostream>
#include <fstream>
#include <cmath>
#include <string>
#include <sstream>      // std::ostringstream

using namespace std;

string resName = "res_alpha";

const double PI = 3.141592653589793238463;

// Set parameters
const int N = 100; // phi range length
const double dt = 1e-6; // timestep 1 microsec
const double a = 2; // termal diffusivity
const double b = 5e3; // K heat power
const double w = 100; // [Hz] omega frequanc
const double c = 5.672e-8; // Stefan-Boltzmann coeffitient
const double theta0 = PI/3; // heat parameter
const double T0 = 273;

double delta_theta = 2*PI/N;
double delta_phi = delta_theta;

int snapshotinterval = 5000; // get result every 5 ms
int stepcount = 1e6; // 1 s timescale

double q(double theta, double phi, double t) {
	// Calcuate heat 
	double qtemp = b*(cos(theta)*cos(theta0)+sin(theta)*sin(theta0)*cos(phi-w*t));
	if(qtemp < 0) qtemp = 0;
	return qtemp;
}

double calc_theta(int i) {
	return double(i)*delta_theta;
}

double calc_phi(int j){
	return double(j)*delta_phi;
}

void euler_step(double** newT, double t){
	double** T = new double*[N/2];
	for(int i = 0; i < N/2; i++) {
		T[i] = new double[N];
		for(int j = 0; j < N; j++)
			T[i][j] = newT[i][j];
	}
	
	for(int i = 0; i < N/2; i++) {
		int k = i+1;
		if(i == (int(N/2)-1) ){
			k = int(N/2) - 1;
		}

		int l = i-1;
		if(i == 0){
			l = 0;
		}
		for(int j = 0; j < N; j++) {
			double temp_calc = 0;
			
			// Periodic boundary at phi
			int m = j+1;
			if(j == N-1) m = 0;

			int n = j-1;
			if(j == 0) n = N-1;

			// Boundary condition at theta is tricky
			int kj = j;
			if(i == (int(N/2)-1) ){
				kj = (int(N/2) + j) % N; // turn back at the pole
			}

			int lj = j;
			if(i == 0){
				lj = (int(N/2) + j) % N; // turn back at the other pole
			}

			temp_calc += a*(T[k][kj]-2*T[i][j]+T[l][lj])/delta_theta/delta_theta;
			if((i != 0) && (i != int(N/2)-1) ){
				temp_calc += a*(T[i][m]-2*T[i][j]+T[i][n])/delta_phi/delta_phi/sin(calc_theta(i))/sin(calc_theta(i));
				temp_calc += a*(T[k][kj]-T[l][lj])/2/delta_phi/tan(calc_theta(i));
			}

			temp_calc += q(calc_theta(i), calc_phi(j), t) - c*T[i][j]*T[i][j]*T[i][j]*T[i][j];
			newT[i][j] = T[i][j] + temp_calc*dt;
		}
	}

	for(int i = 0; i < N/2; i++) {
	    delete T[i];
	}
	delete[] T;
}

int main(int argc, char *argv[]){
	// Initialize tepereture
	// N/2 * N box

	double** T = new double*[N/2]; // ???
	for(int i = 0; i < N/2; i++){
		T[i] = new double[N];
		for(int j = 0; j < N; j++)
			T[i][j] = T0;
	}
	
	int m = 0;
	for(double t = 0; t < stepcount*dt; t += dt) {
		euler_step(T, t);
		if(int(t/dt)%snapshotinterval == 0) {
			ofstream fp;
			ostringstream s;
			s << "snapshots/"+resName+"_" << m << ".dat";
			string tmp(s.str());
			const char *fName = tmp.c_str();
			//cout << fName << endl;
			
			fp.open(fName);
			for(int i = 0; i < N/2; i++) {
				for(int j = 0; j < N; j++) {
					fp << T[i][j] << " ";
				}
				fp << "\n";
			}
			fp.close();
			m += 1;
		}
	}
	
	for(int i = 0; i < N/2; i++) {
	    delete T[i];
	}
	delete[] T;
}
